//+------------------------------------------------------------------+
//|                                                 Liquid Chart.mq5 |
//|                                         Copyright 2014, Decanium |
//|                                          http://www.decanium.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, Decanium"
#property link      "http://www.decanium.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_plots   1
//--- plot ColorCandles
#property indicator_label1  "ColorCandles"
#property indicator_type1   DRAW_COLOR_CANDLES
//---    
#property indicator_color1  clrMediumVioletRed,clrMediumSlateBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//---  
enum ENUM_LC_MODE
  {
   LC_MODE_SS=0,  // Static Shift
   LC_MODE_DSO=1, // Dynamic Shift, just Open
   LC_MODE_DSC=2  // Dynamic Shift, expected Close
  };
//---  
input ENUM_TIMEFRAMES   BaseTF=PERIOD_M1;       // LC Base Period
input int               Depth=100;              // Depth, bars
input ENUM_LC_MODE      inp_LC_mode=LC_MODE_SS; // LC mode
input int               inp_LC_shift=0;         // LC shift
//---  
double ColorCandlesBuffer1[];
double ColorCandlesBuffer2[];
double ColorCandlesBuffer3[];
double ColorCandlesBuffer4[];
double ColorCandlesColors[];
//---  
string symbol;
//--- ,      
ENUM_LC_MODE LC_mode;
int LC_shift = 0;
int LC_coeff = 0;
//---      
#include <liquidchart.mqh>
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---       
   LC_coeff=PeriodSeconds()/PeriodSeconds(BaseTF);
   if(LC_coeff<1)
     {
      Comment("   ");
      return(INIT_PARAMETERS_INCORRECT);
     }
   if(inp_LC_shift<0)
     {
      Comment("  ");
      return(INIT_PARAMETERS_INCORRECT);
     }
   LC_mode=inp_LC_mode;
   if(inp_LC_mode==LC_MODE_SS)
     {
      LC_shift=inp_LC_shift;
      if(LC_shift>=(LC_coeff-1))
         LC_shift=LC_coeff-1;
     }
//--- indicator buffers mapping
   SetIndexBuffer(0,ColorCandlesBuffer1,INDICATOR_DATA);
   SetIndexBuffer(1,ColorCandlesBuffer2,INDICATOR_DATA);
   SetIndexBuffer(2,ColorCandlesBuffer3,INDICATOR_DATA);
   SetIndexBuffer(3,ColorCandlesBuffer4,INDICATOR_DATA);
   SetIndexBuffer(4,ColorCandlesColors,INDICATOR_COLOR_INDEX);
//---  
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);
//---  ,    
   symbol=_Symbol;
//---    
   PlotIndexSetString(0,PLOT_LABEL,symbol+" Open;"+symbol+" High;"+symbol+" Low;"+symbol+" Close");
   IndicatorSetString(INDICATOR_SHORTNAME,"Liquid Chart ("+symbol+")");
//---  
   draw_keys(1,20,5,20,40,2);
//---    
   return(INIT_SUCCEEDED);
  }
//---   rates_total -         OnCalculate
int last_rates_total = 0;
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   last_rates_total=rates_total;
   RefrIndBuffers();
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|               |
//+------------------------------------------------------------------+
void RefrIndBuffers()
  {
   RefreshBuffers(last_rates_total,
                  ColorCandlesBuffer1,ColorCandlesBuffer2,ColorCandlesBuffer3,
                  ColorCandlesBuffer4,ColorCandlesColors);
  }
//+------------------------------------------------------------------+
//|                        |
//+------------------------------------------------------------------+
bool RefreshBuffers(int total,
                    double &buff1[],
                    double &buff2[],
                    double &buff3[],
                    double &buff4[],
                    double &col_buffer[]
                    )
  {
   MqlRates rates[];
   ArrayResize(rates,Depth);
//---
   int copied=0;
   int shift=0;
   if(LC_mode==LC_MODE_SS)
      shift=LC_shift;// 
   else if(LC_mode==LC_MODE_DSO)
      shift=-1;//  (  )
   else if(LC_mode==LC_MODE_DSC)
      shift=-2;//  (  )
   else
      return(false);
//---         
   copied=GetRatesLC(0,Depth,rates,BaseTF,shift);
   if(copied<=0)
     {
      Print(" ");
      return(false);
     }
   LC_shift=shift;
   refr_keys();
//---    
   ArrayInitialize(buff1,0.0);
   ArrayInitialize(buff2,0.0);
   ArrayInitialize(buff3,0.0);
   ArrayInitialize(buff4,0.0);
//---    
   int buffer_index=total-copied;
   for(int i=0;i<copied;i++)
     {
      buff1[buffer_index]=rates[i].open;
      buff2[buffer_index]=rates[i].high;
      buff3[buffer_index]=rates[i].low;
      buff4[buffer_index]=rates[i].close;
      //---
      if(rates[i].open<=rates[i].close)
         col_buffer[buffer_index]=1;//  
      else
         col_buffer[buffer_index]=0;//       
      buffer_index++;
     }
   return(true);
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
void draw_keys(int sub_win,int y_pos,int x_pos,int key_h,int key_w,int key_sp)
  {
   int y=y_pos;
   uchar arrow_dw=242,arrow_up=241;
//--  ,  
   RectLabelXY(sub_win,"key00",x_pos,y,key_w,key_h,clrDarkGray,BORDER_RAISED);
   RectLabelXY(sub_win,"key01",x_pos+key_w+key_sp,y,key_w,key_h,clrDarkGray,BORDER_FLAT);
   RectLabelXY(sub_win,"key02",x_pos+key_w*2+key_sp*2,y,key_w,key_h,clrDarkGray,BORDER_RAISED);
//---  ,  
   y+=key_h+key_sp;
   RectLabelXY(sub_win,"key10",x_pos,y,key_w,key_h,clrDarkGray,BORDER_SUNKEN);
   RectLabelXY(sub_win,"key11",x_pos+key_w+key_sp,y,key_w,key_h,clrDarkGray,BORDER_RAISED);
   RectLabelXY(sub_win,"key12",x_pos+key_w*2+key_sp*2,y,key_w,key_h,clrDarkGray,BORDER_RAISED);
//---    ,  
   y=y_pos+key_h/2;
   SymbXY(sub_win,"txt00",arrow_dw,x_pos+key_w/2,y,14,clrDarkBlue);
   TextXY(sub_win,"txt01","0",x_pos+key_w+key_sp+key_w/2,y,9,clrBlue);
   SymbXY(sub_win,"txt02",arrow_up,x_pos+key_w*2+key_sp*2+key_w/2,y,14,clrDarkBlue);
//---    ,  
   y+=key_h+key_sp;
   TextXY(sub_win,"txt10","SS",x_pos+key_w/2,y,9,clrBlue);
   TextXY(sub_win,"txt11","DSO",x_pos+key_w+key_sp+key_w/2,y,9,clrDarkBlue);
   TextXY(sub_win,"txt12","DSC",x_pos+key_w*2+key_sp*2+key_w/2,y,9,clrDarkBlue);
//---     
   refr_keys();
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
void refr_keys()
  {
//---       
   ObjectSetString(0,"txt01",OBJPROP_TEXT,DoubleToString(LC_shift,0));
//---   ,      ( )
   if(LC_mode==LC_MODE_SS)
     {
      ObjectSetInteger(0,"txt10",OBJPROP_COLOR,clrBlue);
      ObjectSetInteger(0,"key10",OBJPROP_BORDER_TYPE,BORDER_SUNKEN);
     }
   else
     {
      ObjectSetInteger(0,"txt10",OBJPROP_COLOR,clrDarkBlue);
      ObjectSetInteger(0,"key10",OBJPROP_BORDER_TYPE,BORDER_RAISED);
     }
   if(LC_mode==LC_MODE_DSO)
     {
      ObjectSetInteger(0,"txt11",OBJPROP_COLOR,clrBlue);
      ObjectSetInteger(0,"key11",OBJPROP_BORDER_TYPE,BORDER_SUNKEN);
     }
   else
     {
      ObjectSetInteger(0,"txt11",OBJPROP_COLOR,clrDarkBlue);
      ObjectSetInteger(0,"key11",OBJPROP_BORDER_TYPE,BORDER_RAISED);
     }
   if(LC_mode==LC_MODE_DSC)
     {
      ObjectSetInteger(0,"txt12",OBJPROP_COLOR,clrBlue);
      ObjectSetInteger(0,"key12",OBJPROP_BORDER_TYPE,BORDER_SUNKEN);
     }
   else
     {
      ObjectSetInteger(0,"txt12",OBJPROP_COLOR,clrDarkBlue);
      ObjectSetInteger(0,"key12",OBJPROP_BORDER_TYPE,BORDER_RAISED);
     }
//---     ( )
   if((LC_shift==(LC_coeff-1)) || (LC_mode!=LC_MODE_SS))
      ObjectSetInteger(0,"txt02",OBJPROP_COLOR,clrLightGray);
   else
      ObjectSetInteger(0,"txt02",OBJPROP_COLOR,clrDarkBlue);
   if((LC_shift==0) || (LC_mode!=LC_MODE_SS))
      ObjectSetInteger(0,"txt00",OBJPROP_COLOR,clrLightGray);
   else
      ObjectSetInteger(0,"txt00",OBJPROP_COLOR,clrDarkBlue);
  }
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
void RectLabelXY(int sub_win,string ObjName,int x,int y,int dx,int dy,color ObjColor,ENUM_BORDER_TYPE border)
  {
   ObjectDelete(0,ObjName);
   ObjectCreate(0,ObjName,OBJ_RECTANGLE_LABEL,sub_win,0,0,0,0);
   ObjectSetInteger(0,ObjName,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(0,ObjName,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(0,ObjName,OBJPROP_XSIZE,dx);
   ObjectSetInteger(0,ObjName,OBJPROP_YSIZE,dy);
   ObjectSetInteger(0,ObjName,OBJPROP_BORDER_TYPE,border);
   ObjectSetInteger(0,ObjName,OBJPROP_BGCOLOR,ObjColor);
  }
//+------------------------------------------------------------------+
//|                                          |
//+------------------------------------------------------------------+
void SymbXY(int sub_win,string ObjName,char ch,int x,int y,int size,color TextColor)
  {
   ObjectDelete(0,ObjName);
   ObjectCreate(0,ObjName,OBJ_LABEL,sub_win,0,0,0,0);
   ObjectSetInteger(0,ObjName,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(0,ObjName,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(0,ObjName,OBJPROP_COLOR,TextColor);
   ObjectSetInteger(0,ObjName,OBJPROP_FONTSIZE,size);
   ObjectSetString(0,ObjName,OBJPROP_FONT,"Wingdings");
   ObjectSetInteger(0,ObjName,OBJPROP_ANCHOR,ANCHOR_CENTER);
   ObjectSetString(0,ObjName,OBJPROP_TEXT,CharToString(ch));
  }
//+------------------------------------------------------------------+
//|                                            |
//+------------------------------------------------------------------+
void TextXY(int sub_win,string ObjName,string Text,int x,int y,int size,color TextColor)
  {
   ObjectDelete(0,ObjName);
   ObjectCreate(0,ObjName,OBJ_LABEL,sub_win,0,0,0,0);
   ObjectSetInteger(0,ObjName,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(0,ObjName,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(0,ObjName,OBJPROP_COLOR,TextColor);
   ObjectSetInteger(0,ObjName,OBJPROP_FONTSIZE,size);
   ObjectSetString(0,ObjName,OBJPROP_FONT,"Tahoma");
   ObjectSetInteger(0,ObjName,OBJPROP_ANCHOR,ANCHOR_CENTER);
   ObjectSetString(0,ObjName,OBJPROP_TEXT,Text);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---       
   if(id==CHARTEVENT_OBJECT_CLICK)
     {
      if(sparam=="key00" || sparam=="txt00")
        {//  " "
         if(LC_shift>0)
            LC_shift--;
         refr_keys();
         RefrIndBuffers();
         ChartRedraw(0);
        }
      else if(sparam=="key02" || sparam=="txt02")
        {//  " "
         if(LC_shift<(LC_coeff-1))
            LC_shift++;
         refr_keys();
         RefrIndBuffers();
         ChartRedraw(0);
        }
      else if(sparam=="key10" || sparam=="txt10")
        {//  " SS"
         LC_mode=LC_MODE_SS;
         refr_keys();
         RefrIndBuffers();
         ChartRedraw(0);
        }
      else if(sparam=="key11" || sparam=="txt11")
        {//  " DSO"
         LC_mode=LC_MODE_DSO;
         refr_keys();
         RefrIndBuffers();
         ChartRedraw(0);
        }
      else if(sparam=="key12" || sparam=="txt12")
        {//  " DSC"
         LC_mode=LC_MODE_DSC;
         refr_keys();
         RefrIndBuffers();
         ChartRedraw(0);
        }
     }
  }
//+------------------------------------------------------------------+
